function [respondingClasses, responseValues]= getClassifierResponsePrior(classifier, samples, samplePriors, priorFactors)
% samplePriors: indicate the most likely class for each sample
%               0 if prior is unknown
% priorFactors: factors to enlarge clusters of expected classes.
%               should be >= 1 to ensure proper behaviour for samples
%               without prior,

    speed= 1;
    radii= classifier.radii.^2;
    centerClassIds= classifier.centerClassIds;
    classIds= classifier.classIds;
    centers= classifier.centers;
    nCenters= classifier.nCenters;
    nClasses= classifier.nClasses;
    credits= classifier.credits;
        
    nSamples= size(samples, 2);
    respondingClasses= zeros(1, nSamples);
    responseValues= zeros(1, nSamples);
    if nargin == 4
        priorFactors= priorFactors.^2;
        if speed == 1
            
            nPriors= 0;
            minId= min(samplePriors);
            maxId= max(samplePriors);
            priorIds= zeros(1, maxId - minId + 1);
            for id= minId : maxId
                if any(samplePriors == id) && all(priorIds ~= id)
                    nPriors= nPriors + 1;
                    priorIds(nPriors)= id;
                end
            end
            priorIds= priorIds(1:nPriors);
            
            % ASSUMPTION: priorFactors are the same for all samples with
            % same priorId
            radii_= radii(ones(1, nPriors), :);
            for id= 1 : nPriors
                ind= find(samplePriors == priorIds(id), 1);
                radii_(id, centerClassIds == priorIds(id))= radii_(id, centerClassIds == priorIds(id)) * priorFactors(ind);
            end
            
            classResponses= zeros(1, classifier.nClasses);
            for sam= 1 : nSamples
            % calculate responses for all clusters
                sampleMatrix= samples(:, ones(1, nCenters).*sam);
                diffVecs= centers - sampleMatrix;
                distSquared= sum(diffVecs.^2);
                
%                 responses= 1 - distSquared ./ radii_(samplePriors(sam) == priorIds, :);
                responses= (1 - distSquared ./ radii_(samplePriors(sam) == priorIds, :)) .* credits;
                
                responses(responses < 0)= 0;
                for claz= 1 : nClasses
                    cis= centerClassIds == classIds(claz);
                    classResponses(claz)= sum(responses(cis));
                end
                [responseValues(sam), maxClass]= max(classResponses);
                respondingClasses(sam)= classIds(maxClass);
            end
        else % speed
            classResponses= zeros(1, classifier.nClasses);
            for sam= 1 : nSamples
            % calculate responses for all clusters
                sampleMatrix= samples(:, ones(1, nCenters).*sam);
                diffVecs= centers - sampleMatrix;
                distSquared= sum(diffVecs.^2);
                
                factors= ones(1, nCenters); 
                factors(centerClassIds == samplePriors(sam))= priorFactors(sam);
%                 responses= 1 - distSquared ./ (radii .* factors)
                responses= (1 - distSquared ./ (radii .* factors)) .* credits;
                
                for claz= 1 : nClasses
                    cis= centerClassIds == classIds(claz);
                    responses(responses < 0)= 0;
                    classResponses(claz)= sum(responses(cis));
                end
                [responseValues(sam), maxClass]= max(classResponses);
                respondingClasses(sam)= classIds(maxClass);
            end
        end % speed
    else % nargin == 4 (no priors)
        classResponses= zeros(1, classifier.nClasses);
        for sam= 1 : nSamples
        % calculate responses for all clusters
            sampleMatrix= samples(:, ones(1, nCenters).*sam);
            diffVecs= centers - sampleMatrix;
            distSquared= sum(diffVecs.^2);
%             responses= 1 - distSquared ./ radii;
            responses= (1 - distSquared ./ radii) .* credits;
            responses(responses < 0)= 0;
            for claz= 1 : nClasses
                cis= centerClassIds == classIds(claz);
                classResponses(claz)= sum(responses(cis));
            end
            [responseValues(sam), maxClass]= max(classResponses);
            respondingClasses(sam)= classIds(maxClass);
        end
    end % nargin == 4
    
    respondingClasses(responseValues == 0)= 0;
    
    